#!/bin/sh
usage () {
cat<<EOF
usage: configure.sh [-h][-g][-c][-m[0-2]]

-h   print this option summary
-g   include debugging information (default is to optimize)
-c   enable (very) expensive checks (only works with '-g')

The maximum verbosity options '-m[0-2]' are used as follows

-m0  remove code for all verbose messages
-m1  remove code for verbosity level 2 (default without '-g')
-m2  include code for most verbosity levels (default with '-g')
-m3  include code for lowest level messages for logging

This restricts the maximum verbosity at run-time (default 0).
The additional advantage of using '-m0' is that all the verbose
messages and logging code as well as statistics gathering code
goes away and thus the tool becomes slightly faster.
EOF
exit 1
}
debug=no
check=no
maxverbosity=undefined
while [ $# -gt 0 ]
do
  case $1 in
    -g) debug=yes;;
    -c) check=yes;;
    -m0) maxverbosity=0;;
    -m1) maxverbosity=1;;
    -m2) maxverbosity=2;;
    -m3) maxverbosity=3;;
    *) usage;;
  esac
  shift
done
VERSION="`cat VERSION`"
echo "version $VERSION"
COMPILE="gcc -Wall"
if [ $debug = yes ]
then
  COMPILE="$COMPILE -g3"
  [ $maxverbosity = undefined ] && maxverbosity=2
else
  COMPILE="$COMPILE -O3 -DNDEBUG"
  [ $maxverbosity = undefined ] && maxverbosity=1
  check=no
fi
[ $check = no ] && COMPILE="$COMPILE -DNCHECK"
COMPILE="$COMPILE -DMAXVERBOSITY=$maxverbosity"
echo "$COMPILE"
rm -f makefile
sed -e "s,@COMPILE@,$COMPILE," makefile.in >makefile
rm -f config.h
echo "#define VERSION \"$VERSION\"" >>config.h
echo "#define COMPILE \"$COMPILE\"" >>config.h
