#include <assert.h>
#include <ctype.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>

static const char * name;
static int deleted, renamed, reading, writing;
static FILE * input, * output;
static char * backup;

static int * lits, nlits, szlits;

static void die (const char * fmt, ...) {
  va_list ap;
  fputs ("*** sortproof error: ", stderr);
  va_start (ap, fmt);
  vfprintf (stderr, fmt, ap);
  va_end (ap);
  fputc ('\n', stderr);
  if (renamed) {
    if (reading) fclose (input);
    if (writing) fclose (output), unlink (name);
    rename (backup, name);
  }
  exit (1);
}

static void push (int lit) {
  if (szlits == nlits)
    lits = realloc (lits, (szlits = szlits ? 2*szlits : 1) * sizeof *lits);
  lits[nlits++] = lit;
}

static int cmp (const void * p, const void * q) {
  int a = * (int *) p, b = * (int *) q;
  int res = abs (a) - abs (b);
  if (res) return res;
  return a - b;
}

static void sort () {
  int ch, i;
  deleted = renamed = reading = writing = 0;
  input = output = 0;
  backup = 0;
  nlits = szlits = 0;
  lits = 0;
  backup = malloc (strlen (name) + 2);
  sprintf (backup, "%s~", name);
  if (rename (name, backup))
    die ("could not rename '%s' to '%s'", name, backup);
  free (lits);
  if (!(input = fopen (backup, "r")))
    die ("failed to read '%s'", backup);
  if (!(output = fopen (name, "w")))
    die ("failed to write '%s'", name);
  for (;;) {
    ch = getc (input);
    if (isspace (ch)) continue;
    else if (ch == EOF) {
      if (deleted || nlits) die ("terminating zero missing");
      break;
    } else if (ch == 'c' || ch == 'v' || ch == 'p' || ch == 's' || ch == 'a') {
      fputc (ch, output);
      while ((ch = getc (input)) != '\n')
	if (ch == EOF) break;
	else fputc (ch, output);
      if (ch == EOF) break;
      fputc (ch, output);
    } else if (ch == 'd') {
      if (deleted) die ("two 'd' in a row");
      deleted = 1;
    } else {
      int sign, idx, lit;
      if (ch == '-') {
	if (!isdigit ((int)(ch = getc (input))))
	  die ("expected digit after '-'");
	sign = -1;
      } else sign = 1;
      if (!isdigit (ch)) die ("expected digit but got");
      idx = ch - '0';
      while (isdigit (ch = getc (input)))
	idx = 10*idx + (ch - '0');
      ungetc (ch, input);
      lit = sign * idx;
      if (lit) push (lit);
      else {
	qsort (lits, nlits, sizeof *lits, cmp);
	if (deleted) {
	  fputs ("d ", output);
	  deleted = 0;
	}
	for (i = 0; i < nlits; i++)
	  fprintf (output, "%d ", lits[i]);
	fputs ("0\n", output);
	nlits = 0;
      }
    }
  }
  if (fflush (output)) die ("failed to flush '%s'", name);
  if (fclose (output)) die ("failed to close '%s'", name);
  if (fclose (input)) die ("failed to close '%s'", backup);
  if (unlink (backup)) die ("failed to remove '%s'", backup);
  free (backup);
  free (lits);
}

int main (int argc, char ** argv) {
  struct stat buf;
  int i;
  for (i = 1; i < argc; i++) {
    if (!strcmp (argv[i], "-h")) {
      fprintf (stderr, "usage: sortproof [ <proof-or-cnf> ... ]\n");
      exit (1);
    } else if (argv[i][0] == '-')
      die ("invalid option '%s' (try '-h')", argv[i]);
    else if (stat (argv[i], &buf)) die ("can not stat '%s'", argv[i]);
    else name = argv[i], sort ();
  }
  return 0;
}
