#!/bin/sh
cd `dirname $0`
testcases="`basename $0|xargs grep '^drabt [^(]'|wc -l`" echo running $testcases testcases
if [ -f UPDATE ]
then
  echo "will ask for updating golden .out files (UPDATE exists)"
else
  echo "will NOT ask for updating golden .out files (UPDATE missing)"
fi
die () { echo
  echo "*** run-all-tests: after $count test cases: $*" 1>&2
  exit 1
}
asktocopy () {
  echo
  if [ -f $out ]
  then
    echo "*** run-all-tests: diff -c $out $log"
    echo 
    diff -c $out $log
  else
    echo "*** run-all-tests: no '$out'; diff /dev/null $log"
    echo 
    diff /dev/null $log
  fi
  if [ -f UPDATE ]
  then
    echo 
    echo "[To copy '$log' to '$out' enter 'yes']"
    read dummy
    if [ x"$dummy" = xyes ]
    then
      echo "cp $log $out"
      cp $log $out
    else
      echo "not copied"
    fi
  else
    exit 1
  fi
}
erase () {
  if [ -t 1 ]
  then
    echo -n "\r"
    echo -n "          "
    echo -n "          "
    echo -n "          "
    echo -n "          "
    echo -n "          "
    echo -n "          "
    echo -n "          "
    echo -n "        "
#############0123456789
    echo -n "\r"
  else
    echo
  fi
}
drabt () {
  count=`expr $count + 1`
  echo -n "../drabt $*"
  base="`echo $1|sed -e 's,\..*$,,'`"
  log=$base.log
  err=$base.err
  out=$base.out
  rm -f $log $err
  ../drabt $* 1>$log 2>$err
  res=$?
  cat $err >> $log
  echo "exit code $res" >> $log
  if [ -f $out ]
  then
    if diff $out $log 1>/dev/null 2>/dev/null
    then
      passed=`expr $passed + 1`
    else
      failed=`expr $failed + 1`
      asktocopy $out $log
    fi
  else
    failed=`expr $failed + 1`
    asktocopy $out $log
  fi
  erase
}

count=0
passed=0
failed=0

############################################################################
# START OF TEST CASE #######################################################
############################################################################

#---------------------------------------------------------------------------
# PARSE ERRORS
#---------------------------------------------------------------------------

# command line argument parsing tests

drabt noption.cnf --this-is-not-an-option
drabt toomanyfiles.cnf empty.cnf empty.cnf empty.cnf

# some simple existing and non-existing file tests

drabt nocnf.cnf empty.proof
drabt noproof.cnf noproof.proof
drabt nopost.cnf empty.cnf nofile.cnf

# now many parse errors for non-existing headers

drabt head00.cnf empty.proof
drabt head01.cnf empty.proof -I
drabt head02.cnf empty.proof -I
drabt head03.cnf empty.proof -I
drabt head04.cnf empty.proof -I
drabt head05.cnf empty.proof -I
drabt head06.cnf empty.proof -I
drabt head07.cnf empty.proof -I
drabt head08.cnf empty.proof -I
drabt head09.cnf empty.proof -I
drabt head10.cnf empty.proof -I
drabt head11.cnf empty.proof -I
drabt head12.cnf empty.proof -I
drabt head13.cnf empty.proof -I
drabt head14.cnf empty.proof -I
drabt head15.cnf empty.proof -I
drabt head16.cnf empty.proof -I

# 'd' line issues

drabt deof.cnf deof.proof
drabt dnot1stcol.cnf dnot1stcol.proof
drabt dd.cnf dd.proof --force-non-binary
drabt dnozero.cnf dnozero.proof --force-non-binary
drabt dnospace.cnf dnospace.proof --force-non-binary
drabt dnominusordigit.cnf dnominusordigit.proof --force-non-binary
drabt doubled.cnf doubled.proof

# other global parse errors

drabt nozero.cnf empty.proof -I
drabt commenteof.cnf empty.proof
drabt slinenoeof.cnf slinenoeof.proof
drabt slinenotfirstchar.cnf slinenotfirstchar.proof
drabt vlinenoeof.cnf vlinenoeof.proof
drabt vlinenotfirstchar.cnf vlinenotfirstchar.proof

# further literal issues

drabt nodigitafterminus.cnf empty.proof
drabt minuszero.cnf empty.proof

# unexpected characters

drabt unexphead.cnf empty.proof
drabt unexpnohead.cnf empty.proof

drabt nonprint1.cnf
drabt nonprint2.cnf
drabt nonprint3.cnf
drabt nonprint4.cnf
drabt nonprint5.cnf
drabt nonprint6.cnf

# too large indices

drabt varex0.cnf empty.proof
drabt varex1.cnf empty.proof

# number of clauses does not match

drabt toomanyclauses.cnf empty.proof
drabt clausemissing.cnf empty.proof
drabt clausesmissing.cnf empty.proof

# no header warning

drabt headwrn.cnf empty.proof
drabt headwrn2.cnf empty.proof

# invalid 's' line
drabt invalidsline.cnf invalidsline.proof

# d clauses in CNFs

drabt dclauseinpre.cnf empty.proof -I
drabt dclauseinpost.cnf empty.proof dclause.cnf -I

# header in proof

drabt headerinproof1.cnf empty.cnf -S
drabt headerinproof2.cnf empty.cnf -S --pedantic
drabt headerinproof3.cnf empty.cnf -S --ignore-warnings

#---------------------------------------------------------------------------
# BUGGY PROOFS
#---------------------------------------------------------------------------

drabt empty.cnf empty.proof -L
drabt impl00.pre empty.proof impl00.post -L
drabt proof00.pre proof00.proof -M
drabt proof01.pre proof01.proof -M -I
drabt proof02.pre proof02.proof -M -I
drabt impl01.pre impl01.proof impl01.post -M

drabt buggy000.pre buggy000.proof -M

#---------------------------------------------------------------------------
# SIMPLE PROOFS
#---------------------------------------------------------------------------

drabt emptypassing.cnf empty.proof empty.cnf
drabt unit4.cnf emptyclause.proof 
drabt unit5.cnf emptyclause.proof 
drabt unit6.cnf emptyclause.proof 
drabt unit7.cnf emptyclause.proof 
drabt unit8.cnf emptyclause.proof 

drabt sat2proof1.cnf sat2.proof1
drabt sat2proof2.cnf sat2.proof2

drabt sat3proof1.cnf sat3.proof1
drabt sat3proof2.cnf sat3.proof2
drabt sat3proof3.cnf sat3.proof3
drabt sat3proof4.cnf sat3.proof4
drabt sat3proof5.cnf sat3.proof5

drabt unit1twice.cnf unit1twice.proof -S

drabt simple000.pre simple000.proof

#---------------------------------------------------------------------------
# BINARY PROOF FORMAT
#---------------------------------------------------------------------------

drabt bin00.cnf bin00.proof
drabt bin01.cnf bin01.proof -S
drabt bin02.cnf bin02.proof -S
drabt bin03.cnf bin03.proof -S
drabt bin04.cnf bin04.proof -S
drabt bin05.cnf bin05.proof -S
drabt bin06.cnf bin06.proof
drabt bin07.cnf bin07.proof
drabt bin08.cnf bin08.proof
drabt bin09.cnf bin09.proof
drabt bin10.cnf bin10.proof
drabt bin11.cnf bin11.proof
drabt bin12.cnf bin12.proof
drabt bin13.cnf bin13.proof
drabt bin14.cnf bin14.proof
drabt bin15.cnf bin15.proof
drabt bin16.cnf bin16.proof
drabt bin17.cnf bin17.proof

# pretty expensive, checks fifth byte in binary literal encoding:
#
# drabt bin18.cnf bin18.proof -S

#---------------------------------------------------------------------------
# REGRESSIONS
#---------------------------------------------------------------------------

drabt regr0000.cnf regr0000.proof -S
drabt regr0001.cnf regr0001.proof -S
drabt regr0002.cnf regr0002.proof -S
drabt regr0003.cnf regr0003.proof -S
drabt regr0004.cnf regr0004.proof -S

############################################################################
# END OF TEST CASES ########################################################
############################################################################

if [ $passed = $count ]
then
  echo "all $count test cases passed (zero failed)"
elif [ $passed = 0 ]
then
  echo "no test cases passed out of $count ($failed failed)"
else
  echo "only $passed test cases passed out of $count ($failed failed)"
fi
ignored="`basename $0|xargs grep '^#[ 	]*drabt [^(]'|wc -l`"
if [ $ignored -gt 0 ]
then
  echo "ignored $ignored test cases"
else
  echo "no ignored test cases"
fi
